
// EMS Class - test case
// Copyright 1993 Scott Diorio
// All rights reserved


#include <process.h>
#include <conio.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <alloc.h>
#include <fcntl.h>
#include <io.h>
#include <assert.h>

#include "emsclass.hpp"
#define CHUNK 16384
#define ON 1
#define OK 0
#define CHARBUFSIZE 65532

int test0(void);
int test1(void);
int test2(void);
int test3(void);
int test4(void);
int test5(void);

EmsKernel EMS(ON,2);   // set debugging ON, errorlevel to 2

void main(void)
{

	int pages;
	if(( pages=EMS.SystemPagesFree() ) ==-1)
		exit(-1);

	cout << "System EMS Pages Free: " << pages << endl;
	cout << "EMS Pages Free:  " << EMS.AppPagesFree() << endl;;

	EMS.Alloc(63); // Alloc 50 - 16K pages
	cout << "EMS Pages Free:  " << EMS.AppPagesFree() << endl;;

	test0();

	// test 1
	if(test1())
		{
			cout << "test 1 failed\n";
			exit(-1);
		}
	cout << "test 1 passed\n";
	cout << "\n" << farcoreleft() << endl;

	// test 2
	if(test2())
		{
			cout << "test 2 failed\n";
			exit(-1);
		}
	cout << "test 2 passed\n";
	cout << "\n" << farcoreleft() << endl;

	// test 3
	if(test3())
		{
			cout << "test 3 failed\n";
			exit(-1);
		}
	cout << "test 3 passed\n";
	cout << "\n" << farcoreleft() << endl;


	// test 4
	if(test4())
		{
			cout << "test 4 failed\n";
			exit(-1);
		}
	cout << "test 4 passed\n";
	cout << "\n" << farcoreleft() << endl;

	// test 5
	if(test5())
		{
			cout << "test 5 failed\n";
			exit(-1);
		}
	cout << "test 5 passed\n";
	cout << "\n" << farcoreleft() << endl;

}


/////////////////////////////////////////////////////////////////////////
// Test page allocation and kernel mapping

int test0(void)
{
	EMSbuf *a1,*a2,*a3,*a4;

	// create 3 contiguous EMSbuf objects
	a1=new EMSbuf(52000);
	EMS.DumpAll();
	a2=new EMSbuf(32000);
	EMS.DumpAll();
	a3=new EMSbuf(52000);
	EMS.DumpAll();

	// delete a2 so a hole is made in the available mapping structure
	delete(a2);
	EMS.DumpAll();

	// allocate an object that is larger than the available open space
	// freed by deleting a2. This show that an EMSbuf does not need to
	// be stored in contiguous EMS blocks
	a2=new EMSbuf(52000);
	EMS.DumpAll();


	a3->Resize(10000);
	EMS.DumpAll();

	a4=new EMSbuf(52000);
	EMS.DumpAll();

	a4->Resize(32000);
	EMS.DumpAll();

	a3->Resize(865000);
	EMS.DumpAll();

	// delete objects
	delete(a1);
	EMS.DumpAll();

	delete(a2);
	EMS.DumpAll();

	delete(a3);
	EMS.DumpAll();

	delete(a4);
	EMS.DumpAll();

	return(0);
}

/////////////////////////////////////////////////////////////////////////
// Test operator =, copy constructor
int test1(void)
{
	// test operator =
	EMSbuf A(10);
	memset(A,'A',10);
	EMSbuf B(1);
	memset(B,'B',1);
	A=B;                   // set A to B
	cout << B << endl;     //
	cout << A << endl;     //

	// test the copy constructor
	EMSbuf C(10);
	memset(C,'*',10);
	EMSbuf D=C;
	cout << D << endl;
	return(0);
}

/////////////////////////////////////////////////////////////////////////
// Tests - usage of 0 & 1 length buffers,
//       - Resize() function
//

int test2(void)
{
	EMSbuf A(0);
	memset(A,'A',1);   // should print error
	A[1]='*';          // should print error
	cout << A << endl; // should print error

	EMSbuf B(1);
	memset(B,'*',1);    // ok
	memset(B,'*',2); 	  // should print error
	cout << B << endl;  //

	A.Resize(1000);
	memset(A,'X',1001); // should print error
	memset(A,'X',1000); // ok
	cout << A << endl;
	return(0);
}

/////////////////////////////////////////////////////////////////////////
// Test single character manipulation in EMS buffer
int test3(void)
{
	// create two EMS buffers
	EMSbuf A(100000);
	EMSbuf B(100000);

	unsigned long t;

	// assign random values to first buffer
	for(t=0;t<=100000;t++)
		A[t]=random(255);

	// copy buffer A to buffer B one character at a time
	for(t=0;t<=100000;t++)
		B[t]=A[t];

	// loop through and test buffers A and B for equivilance
	for(t=0;t<100000;t++)
		assert( B[t] == A[t] );

	// create two character buffers
	static unsigned char *a=new char[CHUNK];
	static unsigned char *b=new char[CHUNK];

	// loop through and test buffers A and B for equivilance
	for(unsigned long q=0;q<90000;q+=CHUNK)
		{
			memcpy(a,A+q,CHUNK);
			memcpy(b,B+q,CHUNK);
			for(t=0;t<CHUNK;t++)
				assert(a[t]==b[t]);
		}
	delete(a);
	delete(b);
	return(OK);
}

/////////////////////////////////////////////////////////////////////////
// Tests 1) Setting EMS buffers
//       2) Copying between character buffers and EMS buffers
//
// The results are displayed on screen
//
// Set breakpoints to follow example

#define TESTSIZE 40000

int test4(void)
{
	unsigned t;
	char ch;

	EMSbuf *A;
	A=new EMSbuf(TESTSIZE);
	memset( *A, '1', TESTSIZE);     // friend function to EMSbuf

	// display results of memset
	cout << "Press <spacebar> to see results, any other key to skip\n";
	cout << "Be patient, it takes a few seconds to display " << TESTSIZE << " characters\n\n";
	ch=getch();
	if(ch==32)
		cout << *A << endl;

	char *tmp;
	tmp=new unsigned char[TESTSIZE];
	memset(tmp,'2',TESTSIZE);      // normal memset

	memcpy( *A,tmp,TESTSIZE);
	// display results of memcpy
	cout << "Press <spacebar> to see results, any other key to skip\n";
	cout << "Be patient, it takes a few seconds to display " << TESTSIZE << " characters\n\n";
	ch=getch();
	if(ch==32)
		cout << *A << endl;

	delete(A);

////////////////////////////////////////////////////////////////////
//
// This example show how you can access an EMS buffer like a char
// array using the overloaded operator '+'
//

	EMSbuf B(TESTSIZE);

	memset(B,'*',TESTSIZE);
	memset(tmp,'!',TESTSIZE);

	// Copy 10,000 characters from character buffer 'tmp'
	// starting 20 characters into EMS buffer B
	//
	//
	//         overloaded operator
	//        /
	memcpy( B+20, tmp, 10000);

	cout << "Press <spacebar> to see results, any other key to skip\n";
	cout << "Be patient, it takes a few seconds to display " << TESTSIZE << " characters\n\n";
	ch=getch();
	if(ch==32)
		cout << B << endl;

	delete(tmp);
	return(OK);
}


///////////////////////////////////////////////////////////////////////////
//
// This function tests the correct setting of EMS buffers starting at every
// possible offset location. The range is from 0 to 131072, which spans
// the size of 2 64K page frame buffers. CHARBUFSIZE bytes are written to
// the buffer on each test. The EMSbuf is created and deleted on each
// successive test.
//
// This test uses "charbuf" which holds a series of random numbers.
// The charbuf is copied to the EMSbuf and then a comparison is made to
// test whether the copy was successful or not.
//
// This test takes appoximately 45 minutes on a 486-50
//
//

int test5(void)
{
cout << "This function tests the correct setting of EMS buffers starting at every\n";
cout << "possible offset location. The range is from 0 to 131072, which spans\n";
cout << "the size of 2 64K page frame buffers. CHARBUFSIZE bytes are written to\n";
cout << "the buffer on each test. The EMSbuf is created and deleted on each\n";
cout << "successive test.\n";
cout << "\n";
cout << "This test uses 'charbuf' which holds a series of random numbers.\n";
cout << "The charbuf is copied to the EMSbuf and then a comparison is made to\n";
cout << "test whether the copy was successful or not.\n";
cout << "\n";
cout << "This test takes appoximately 45 minutes on a 486-50\n";
cout << "\n";

	cout << "PRESS <SPACEBAR> to run test, any other key to abort test\n";
	int ch;
	ch=getch();
	if(ch!=32)
		return(0);

	EMSbuf *emsbuf;
	char *charbuf;
	charbuf=new char[CHARBUFSIZE];
	if(!charbuf)
		exit(-1);

	// fill test buffer with unique values
	for(unsigned i=0;i<CHARBUFSIZE+1;i++)
		charbuf[i]=random(255);

	long memcnt;
	long destoffset;
	int cnt=0;
	cout << "processing";
	memcnt=farcoreleft();
	for(destoffset=0;destoffset<131072;destoffset++)
		{
			emsbuf=new EMSbuf(destoffset+CHARBUFSIZE);
			memcpy( *emsbuf+destoffset,charbuf,CHARBUFSIZE); // ok
			assert( ! memcmp( *emsbuf+destoffset,charbuf,CHARBUFSIZE) );
			delete(emsbuf);
			if(cnt==100)
				{
					cnt=0;
					cout << ".";
				}
			cnt++;
		}
	if(farcoreleft() != memcnt)
		return(-1);
	delete(charbuf);
}